//+------------------------------------------------------------------+
//|                                                    ATR - mtf.mq4 |
//|                                                 Copyright  2010 |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 YellowGreen
#property indicator_width1 3

//--- input parameters
extern string TimeFrame       = "Current time frame";
extern int    ATRPeriod		  = 12;
//--- buffers
double ATRBuffer[];

// Globals
string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(0,ATRBuffer);
//----
   ATRPeriod = MathMax(ATRPeriod,1);
   
   //
   //
   indicatorFileName = WindowExpertName();
   calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
   returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
   timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   IndicatorShortName(timeFrameToString(timeFrame)+" ("+ATRPeriod+")");
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int    counted_bars=IndicatorCounted();
//----
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1-ATRPeriod);
           if (returnBars) { ATRBuffer[0] = MathMin(limit+1,Bars-1-ATRPeriod); return(0); }

   //
   //
   if (calculateValue || timeFrame == Period())
   {
      for(i=limit; i>=0; i--)
      {
         ATRBuffer[i] = iATR(NULL,0,ATRPeriod,i);
      }            
      return(0);
   }      

   //
   //
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         ATRBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",ATRPeriod,0,y);
   }
//----
   return(0);
  }
//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}
//+------------------------------------------------------------------+